import unittest


class Tests(unittest.TestCase):

    def test_unit_test(self):
        got = "hello"
        expected = "hello"
        self.assertEqual(got, expected)


if __name__ == '__main__':
    unittest.main()



from flask import Flask, request, render_template, jsonify, abort
import multiprocessing
from multiprocessing import Pool
import time
import os
import numpy as np
import sys
import urllib
import webbrowser
import string
import glob
import random
from metamusic.fetcher import process_init
from metamusic.model import db, fetcher_database
import random

from pathlib import Path

import os

app = Flask(__name__, static_folder="./static/dist",
            template_folder="./static")

database_name = ''.join(random.choices(
    string.ascii_uppercase, k=10))
db_path = os.path.join(Path.home(), '.metamusic')
if not os.path.exists(db_path):
    os.mkdir(db_path)
for i in glob.glob(os.path.join(db_path, '*.db')):
    os.remove(i)
app.config['SQLALCHEMY_DATABASE_URI'] = 'sqlite:///' + \
    db_path + '/' + database_name + '.db'
app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
app.secret_key = "thisisyunik"
db.init_app(app)


@app.route('/')
def index():

    return render_template('index.html')


@app.route("/process", methods=["GET"])
def process():

    path = request.args['path']
    if not os.path.exists(path):
        time.sleep(.3)
        return render_template("nofile.html")
    total_songs = 0

    if not os.path.isfile(path):
        p = Pool()
        folders = []
        for root, dirs, files in os.walk(path):

            results = p.map(f, files)
            numbers = np.sum(np.array(results) > 0)
            if numbers:
                folders.append(root)
            total_songs += numbers
        p.close()
        p.join()

        t = multiprocessing.Process(
            target=process_init, args=(path, app, db, folders))
    else:
        total_songs = 1
        folders = [os.path.dirname(path)]
        t = multiprocessing.Process(
            target=process_init, args=(path, app, db, folders))
    t.daemon = True
    t.start()
    time.sleep(0.4)
    return render_template("process.html", totalsongs=total_songs)


@app.route('/fetch/<int:no>')
def fetch(no):
    return_data = fetcher_database.query.filter_by(uid=no).first()

    while return_data is None:
        time.sleep(0.01)
        return_data = fetcher_database.query.filter_by(uid=no).first()
    if not return_data.status:
        return abort(404)
    return jsonify(trackname=return_data.trackname, tracknumber=return_data.tracknumber, albumname=return_data.albumname, image_url=return_data.image_url, releasedate=return_data.releasedate,
                   genre=return_data.genre, artistname=return_data.artistname, uid=return_data.uid, loading=True)


def f(n):
    if (n.endswith('.mp3')):
        return 1
    else:
        return 0


def run():
    if len(sys.argv) >= 2:

        if os.path.exists(sys.argv[1]):
            path = os.path.abspath(sys.argv[1])
            if os.path.isdir(path):
                url = 'http://127.0.0.1:5000/process?path=' + \
                    urllib.parse.quote_plus(path)
                webbrowser.open(url)
                app.run(threaded=True)
            else:
                folders = [os.path.dirname(path)]
                process_init(path, app, db, folders)
        else:
            raise ValueError("The path/ file you entered doesn't exit")
    else:
        webbrowser.open('http://127.0.0.1:5999/')
        app.run(threaded=True, port=5999)



from flask_sqlalchemy import SQLAlchemy
from werkzeug import generate_password_hash, check_password_hash

import datetime
import json
db = SQLAlchemy()


class fetcher_database(db.Model):
    __tablename__ = 'fetcher'
    uid = db.Column('id', db.Integer, primary_key=True)
    trackname = db.Column('trackname', db.Integer)
    tracknumber = db.Column('tracknumber', db.Integer)
    image_url = db.Column('image_url', db.Text)
    artistname = db.Column('artistname', db.String(50))
    albumname = db.Column('albumname', db.String(50))

    releasedate = db.Column('releasedate', db.Integer)
    genre = db.Column('genre', db.String(50))

    status = db.Column('status', db.Boolean())



import os
import requests
from metamusic.model import fetcher_database
import eyed3
import asyncio
from bs4 import BeautifulSoup
import re
import threading
import time
import glob
from mutagen.id3 import ID3, TYER

lyrics_bool = False

base_url = "http://api.genius.com"
url = 'https://itunes.apple.com/search'
headers = {
    'Authorization': 'Bearer dp7sB4-Li2skNwHMdBuXz2yQYKm2moTTW7aVLI1yLBxVnB479rf3HFDJbB9hoDe0'}
search_url = base_url + "/search"


async def main(song_name):
    global base_url, url, headers, search_url
    data = {'q': song_name}

    loop = asyncio.get_event_loop()
    try:
        itunes = loop.run_in_executor(None, lambda: requests.get(url, params={
            "term": song_name, "media": "music", "entity": "song", "limit": 1}).json())

        genius = loop.run_in_executor(None, lambda: requests.get(
            search_url, params=data, headers=headers).json())
        response1 = await itunes
        response2 = await genius
    except Exception:

        print("Error occured when fetching data from servers")
        return 0
    return [response1, response2]


def sync_data(data, image_url, lyrics_url, song_path):
    global lyrics_bool, base_url, url, headers, search_url
    if lyrics_bool:  # if itunes and genius dont match the song and artist
        try:
            data_ = {'q': data['trackName'] + ' ' + data['artistName']}
            print(data['trackName'] + ' ' + data['artistName'])
            genius = requests.get(search_url, params=data_,
                                  headers=headers).json()

            lyrics_url = genius["response"]["hits"][0]["result"]['url']
            print('inside')
        except IndexError:
            print("indexerror")
    page = requests.get(lyrics_url)
    html = BeautifulSoup(page.text, "html.parser")
    lyrics_ = html.find("div", class_="lyrics").get_text()

    tags = ID3()
    tags['TYER'] = TYER(encoding=3, text=data["releaseDate"][0:4])  # year
    tags.save(song_path)
    audiofile = eyed3.load(song_path)
    eyed3.log.setLevel("ERROR")
    audiofile.tag.artist = data["artistName"]
    audiofile.tag.album = data["collectionName"]
    audiofile.tag.album_artist = data["artistName"]
    audiofile.tag.title = data["trackName"]
    audiofile.tag.track_num = data["trackNumber"]
    audiofile.tag.release_date = data["releaseDate"][0:4]

    audiofile.tag.genre = data["primaryGenreName"]
    img = requests.get(image_url).content

    audiofile.tag.images.set(3, img, "image/jpeg")

    audiofile.tag.lyrics.set(lyrics_)

    audiofile.tag.save()

    print(song_path, data["artistName"])


def process_init(path, app, db, folders):
    song_no = 0
    total_managed = 0
    isFile = False
    timeit = time.time()
    if os.path.isfile(path):
        isFile = True
    with app.app_context():
        db.create_all()
        print(os.path.dirname(path))
        for root in folders:

            for i in os.listdir(root):

                if i != os.path.basename(path) and isFile:

                    continue
                ext = os.path.splitext(i)[1]
                if i.endswith('.mp3'):

                    temp = os.path.splitext(i)[0]

                    i = re.sub(re.escape(ext), '', i)
                    i = re.sub(r'[^\w^,]', ' ', i)
                    i = re.sub(r'[_]', ' ', i)
                    i = re.sub(r'^[0-9]+[ _\-][0-9]*', '', i)
                    i = re.sub(r' \d{2,}', '', i)
                    i = re.sub(r'[^\x00-\x7F]+', '', i).strip()
                    if i != temp:
                        os.rename(os.path.join(root, temp + ext),
                                  os.path.join(root, i + ext))
                        print("{0}{2} renamed to {1}{2}".format(
                            temp, i, ext))

                    loop = asyncio.new_event_loop()
                    asyncio.set_event_loop(loop)
                    datas = loop.run_until_complete(main(i))
                    if datas == 0:
                        song_no += 1
                        continue
                    try:
                        global lyrics_bool

                        data = datas[0]['results'][0]
                        genius_data = datas[1]["response"]["hits"][0]["result"]
                        if data['artistName'].lower().strip() == genius_data['primary_artist']['name'].lower().strip():

                            image_url = genius_data['song_art_image_thumbnail_url']
                            lyrics_bool = False
                        else:
                            image_url = data['artworkUrl100']
                            lyrics_bool = True

                    except IndexError:
                        print("Data related to {} was not found".format(i))
                        fetched_data = fetcher_database(
                            uid=song_no, status=False)
                        db.session.add(fetched_data)
                        db.session.commit()
                        song_no += 1
                        continue
                    lyrics_url = genius_data['url']
                    releasedate = data["releaseDate"][0:4]
                    try:

                        fetched_data = fetcher_database(
                            trackname=data["trackName"], uid=song_no, tracknumber=data["trackNumber"], image_url=image_url, artistname=data["artistName"], albumname=data["collectionName"], releasedate=releasedate, genre=data["primaryGenreName"], status=True)
                        db.session.add(fetched_data)
                        db.session.commit()
                        total_managed += 1
                    except Exception:

                        print("Already in database")
                        song_no += 1
                        continue
                    song_no += 1
                    t = threading.Thread(target=sync_data, args=(
                        data, image_url, lyrics_url, os.path.join(root, i + ext)))
                    t.daemon = True
                    t.start()
                    if isFile:
                        break

        time.sleep(4)
        db.session.query(fetcher_database).delete()
        db.session.commit()
        print("{} out of {} songs were managed".format(total_managed, song_no))
        print((time.time() - timeit) - 3)



import setuptools
import sys
import os
from setuptools.command.install import install
from pathlib import Path


class CustomInstallCommand(install):
    """Customized setuptools install command - prints a friendly greeting. for now"""

    def run(self):
        install.run(self)

        if sys.platform == 'linux':

            home = os.path.join(Path.home(), '.metamusic')
            if not os.path.exists(home):
                os.mkdir(home)
            os.system(f'cp ./metamusic/metamusic.png {home}')
            desk = f'''\
[Desktop Entry]
Version=1.0
Type=Application
Terminal=false
Name=Meta-Music
Exec=meta-music
Icon={home}/metamusic.png
StartupNotify=true
Categories=Application;'''
            with open('./metamusic/meta-music.desktop', 'w') as f:
                f.write(desk)

            os.system(
                'sudo cp ./metamusic/meta-music.desktop /usr/share/applications/')
            print("Adding desktop icon....")


def parse_requirements(requirements):
    # load from requirements.txt
    with open(requirements) as f:
        lines = [l for l in f]
        # remove spaces
        stripped = map((lambda x: x.strip()), lines)
        # remove comments
        nocomments = filter((lambda x: not x.startswith('#')), stripped)
        # remove empty lines
        reqs = filter((lambda x: x), nocomments)
        return reqs


REQUIREMENTS = parse_requirements("requirements.txt")
setuptools.setup(
    name="MetaMusic",
    version="1.1.3",
    url="https://github.com/unique1o1/Meta-Music",
    author="Yunik Maharjan",
    author_email="yunik.maharjan@icloud.com",
    license='MIT',
    description="Metamusic",
    platforms="Linux, MacOS, Windows",
    long_description=open('README.md').read(),
    include_package_data=True,
    install_requires=[i for i in REQUIREMENTS],

    packages=setuptools.find_packages(),
    classifiers=[
        'Environment :: Console',
        'Programming Language :: Python :: 3.6'
    ],
    cmdclass={
        'install': CustomInstallCommand,
    },
    entry_points="""
    [console_scripts]
    meta-music=metamusic.app:run
    """,
)
